<?php

############################################################################
#
# Copyright [2020] [Mihály Héder]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
############################################################################

function base64url_encode($data) {
    return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
}

function cl_escape($str) {
    return str_replace("'","&quot;",$str);
}

include "init.php";

$claim_id = $argv[1];

$claim = get_claim_data($claim_id)[0];

$title = $claim["title"]; 
$content = $claim["content"];

$signers = explode(',',$claim["signers"]);

$claimdate = $claim['created_at'];

$subject_id = $claim['subject'];
$subjectName = get_subject_name_by_id($subject_id);

$claim_json =
'{' ."\n".
'    "@context":['."\n".
'       "https://www.w3.org/2018/credentials/v1",'."\n".
'       "https://educlaims.ponens.org/claim"'."\n".
'    ],'."\n".
//'    "id":"http://example.edu/credentials/1872",'."\n".
'    "type":['."\n".
'       "VerifiableCredential",'."\n".
'       "FreeTextClaim"'."\n".
'    ],'."\n".
'    "issuer":"https://educlaims.ponens.org",'."\n".
'    "issuanceDate":"' . date(DATE_ATOM) . '",'."\n".
'    "credentialSubject":{'."\n".
'       "name":"' . cl_escape($subjectName) . '",'."\n".
'       "freeTextClaim":"' . cl_escape($content) . '",'."\n".
'       "signer":['."\n";

$c = count($signers);
foreach ($signers as $i => $signer) {
        $claim_json .= '          {'."\n".
            '              "name":"' . cl_escape(get_user_display_string($signer)) . '",'."\n".
            '              "date":"' . cl_escape(get_signature_date($claim_id,$signer)) . '"'."\n".
            '          }';
        if($i < ($c-1)) {
            $claim_json .=',';
        }
        $claim_json .= "\n";
}
$claim_json .=
'       ]'."\n".
'    }'."\n".
'}';

//build the headers
$headers = ['alg'=>'RS256','typ'=>'JWT'];
$headers_json = json_encode($headers);
$headers_encoded = base64url_encode($headers_json);

$payload_encoded = base64url_encode($claim_json);

//build the signature
$key = file_get_contents($GLOBALS['app']['keyfile']);
openssl_sign($payload_encoded, $signature, $key, OPENSSL_ALGO_SHA256); 
$signature_encoded = base64url_encode($signature);

exec("docker run moul/drunken-bishop " . $signature_encoded,$bishop);

// Include the main TCPDF library (search for installation path).
require_once('tcpdf.php');

// create new PDF document
$pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);

// set document information
$pdf->SetCreator(PDF_CREATOR);
$pdf->SetAuthor('eduCLAIMS service');
$pdf->SetTitle($title);
$pdf->SetSubject('eduCLAIMS claim' . date("Y-M-d H:i:s") );

// set default header data
$pdf->SetHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, PDF_HEADER_TITLE, PDF_HEADER_STRING);

// set header and footer fonts
$pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

// set default monospaced font
$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

// set margins
$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

// set auto page breaks
$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

// set image scale factor
$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

$certificate = $GLOBALS['app']['pemfile'];

// set additional information
$info = array(
	'Name' => 'eduCLAIMS document',
	'Location' => 'eduCLAIMS service Budapest',
	'Reason' => 'eduCLAIMS Proof of Concept',
	'ContactInfo' => 'https://educlaims.ponens.org',
	);

// set document signature
$pdf->setSignature($certificate, $certificate, 'educlaims', '', 2, $info);

// set font
$pdf->SetFont('helvetica', '', 12);

//page1
$pdf->AddPage();
$text = "<h1>" . $title . "</h1>";
$text .= nl2br($content);
$text .= '<h2>Signers</h2>';

foreach ($signers as $signer) {
    $text .= "&nbsp;&nbsp;&nbsp;" . get_user_display_string($signer) . " at " . get_signature_date($claim_id,$signer) . "<br><br>";
}
$pdf->writeHTML($text, true, 0, true, 0);

$pdf->SetFont('helvetica', '', 8);
//page2
$pdf->AddPage();
$text = "<h1>Machine-readable representation</h1><br>";
$text .= '<pre>';
$text .= $claim_json;
$text .= '</pre>';
$pdf->writeHTML($text, true, 0, true, 0);

$pdf->SetFont('helvetica', '', 12);
$pdf->AddPage();
$text = "<h1>Digital Signature</h1>";
$text .= "<h2>Signature (JWS) Parameters</h2>";
$text .= '<pre>';
$text .= $headers_json;
$text .= '</pre>';
$text .= "<h2>Signature value</h2>";
$text .= $signature_encoded;
$text .= '<pre>';
$text .= "<h2>Signature value visualization</h2>";
foreach($bishop as $line) {
    $text.= '                         ';
    $text.= $line."\n";
}
$text .= '</pre>';
$text .= "<h1>How to verify this claim?</h1>";
$text .= "You should <b>never</b> trust an eduCLAIMS pdf <b>without verifying </b>it. <b>Anyone</b> can make a pdf that looks like this.";
$text .= "<br/><br/>";
$text .= "For instructions of verification, please visit: <a href='https://educlaims.ponens.org/doc/verify'>https://educlaims.ponens.org/doc/verify</a> !";
$pdf->writeHTML($text, true, 0, true, 0);

$tmpfname = tempnam("/tmp", "edc");
//Close and output PDF document
$pdf->Output($tmpfname, 'F');

//echo $tmpfname . "\n";
$hash =  system("sha256sum -b " . $tmpfname .  "|cut -d ' ' -f1");
$hashhash = hash('crc32b',$hash,false);

$filename = $hashhash . ".pdf";
$filepath = $GLOBALS['app']['output'] . "/" . $filename;

rename($tmpfname,  $filepath);

chmod($filepath,0644);

mark_claim_signed($claim_id,$hash,$filename);