<?php

############################################################################
#
# Copyright [2020] [Mihály Héder]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
############################################################################

include "init.php";

// Include the main TCPDF library (search for installation path).
require_once('tcpdf.php');

// extend TCPF with custom functions
class MYPDF extends TCPDF {

    // Load table data from file
    public function LoadData($file) {
        // Read file lines
        $lines = file($file);
        $data = array();
        foreach($lines as $line) {
            $data[] = explode(';', chop($line));
        }
        return $data;
    }

    // Colored table
    public function ColoredTable($header,$data) {
        // Colors, line width and bold font
        $this->SetFillColor(255, 0, 0);
        $this->SetTextColor(255);
        $this->SetDrawColor(128, 0, 0);
        $this->SetLineWidth(0.3);
        $this->SetFont('', 'B');
        // Header
        $w = 20;
        $num_headers = count($header);
        for($i = 0; $i < $num_headers; ++$i) {
            $this->Cell($w, 7, $header[$i], 1, 0, 'C', 1);
        }
        $this->Ln();

        // Data
        $fill = 0;
        foreach($data as $row) {
            if ($fill) {
                $this->SetFillColor(128, 128, 128);
                $this->SetTextColor(255);
            } else {
                $this->SetFillColor(255, 255, 255);
                $this->SetTextColor(0);
            }
            
            $this->SetFont('', 'B');
            
            $this->Cell($w, 6, $row[0], 'LR', 0, 'C', true);
            
            $this->SetFillColor(224, 235, 255);
            $this->SetTextColor(0);
            $this->SetFont('');

            $this->Cell($w, 6, $row[1], 'LR', 0, 'C', $fill);
            $this->Cell($w, 6, $row[2], 'LR', 0, 'C', $fill);
            $this->Cell($w, 6, $row[3], 'LR', 0, 'C', $fill);
            $this->Cell($w, 6, $row[4], 'LR', 0, 'C', $fill);
            $this->Cell($w, 6, $row[5], 'LR', 0, 'C', $fill);
            $this->Cell($w, 6, $row[6], 'LR', 0, 'C', $fill);
            $this->Cell($w, 6, $row[7], 'LR', 0, 'C', $fill);
            $this->Cell($w, 6, $row[8], 'LR', 0, 'C', $fill);
            $this->Ln();
            $fill=!$fill;
        }
        $this->Cell(9*$w, 0, '', 'T');
    }
}

// create new PDF document
$pdf = new MYPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);

$codebook_id = $argv[1];

// set document information
$pdf->SetCreator(PDF_CREATOR);
$pdf->SetAuthor('eduCLAIMS codebook');
$pdf->SetTitle($codebook_id);
$pdf->SetSubject('eduCLAIMS codebook' . date("Y-M-d H:i:s") );

// set default header data
$pdf->SetHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, "eduCLAIMS codebook #" . $codebook_id, "Codes for Second Factor");

// set header and footer fonts
$pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

// set default monospaced font
$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

// set margins
$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

// set auto page breaks
$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

// set image scale factor
$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

// set additional information
$info = array(
	'Name' => 'eduCLAIMS codebook',
	'Location' => 'eduCLAIMS service Budapest',
	'Reason' => 'eduCLAIMS Proof of Concept',
	'ContactInfo' => 'https://educlaims.ponens.org',
	);

// set font
$pdf->SetFont('helvetica', '', 12);

$letters = array("##","A","B","C","D","E","F","G","H");

for ($i = 1; $i<=8; $i++) {
    $pdf->AddPage();

    $data = array();
    for($j = 1; $j <= 38; $j++) {
        $row = array();
        array_push($row,sprintf('%02d',$j));
        for($k = 1; $k<=8; $k++) {
            $vr = random_int(0,999);
            $vs = sprintf('%03d', $vr);
            array_push($row,$vs);
            add_codebook_item($codebook_id,$i,$letters[$k],$j,$vs);
        }
        array_push($data,$row);
    }

    $pdf->ColoredTable($letters, $data);
}

$fname = "/opt/out/" . $codebook_id . ".pdf";
//Close and output PDF document
$pdf->Output($fname, 'F');
echo $fname . "\n";