<?php

############################################################################
#
# Copyright [2020] [Mihály Héder]
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
############################################################################

include "init.php";

function base64url_encode($data) {
    return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
}

function cl_escape($str) {
    return str_replace("'","&quot;",$str);
}

// --- load data from DB
$claim_id = $argv[1];

$claim = get_claim_data($claim_id)[0];

$title = $claim["title"]; 
$content = $claim["content"];

$signers = explode(',',$claim["signers"]);

$claimdate = $claim['created_at'];

$subject_id = $claim['subject'];
$subjectName = get_subject_name_by_id($subject_id);
// ---

$claim_json =
'{' ."\n".
'    "@context":['."\n".
'       "https://www.w3.org/2018/credentials/v1",'."\n".
'       "https://educlaims.ponens.org/claim"'."\n".
'    ],'."\n".
//'    "id":"http://example.edu/credentials/1872",'."\n".
'    "type":['."\n".
'       "VerifiableCredential",'."\n".
'       "FreeTextClaim"'."\n".
'    ],'."\n".
'    "issuer":"https://educlaims.ponens.org",'."\n".
'    "issuanceDate":"' . date(DATE_ATOM) . '",'."\n".
'    "credentialSubject":{'."\n".
'       "name":"' . cl_escape($subjectName) . '",'."\n".
'       "freeTextClaim":"' . cl_escape($content) . '",'."\n".
'       "signer":['."\n";

$c = count($signers);
foreach ($signers as $i => $signer) {
        $claim_json .= '          {'."\n".
            '              "name":"' . cl_escape(get_user_display_string($signer)) . '",'."\n".
            '              "date":"' . cl_escape(get_signature_date($claim_id,$signer)) . '"'."\n".
            '          }';
        if($i < ($c-1)) {
            $claim_json .=',';
        }
        $claim_json .= "\n";
}
$claim_json .=
'       ]'."\n".
'    }'."\n".
'}';

echo $claim_json . "\n";

//build the headers
$headers = ['alg'=>'RS256','typ'=>'JWT'];
$headers_encoded = base64url_encode(json_encode($headers));

$payload_encoded = base64url_encode($claim_json);

//build the signature
$key = file_get_contents($GLOBALS['app']['keyfile']);
openssl_sign($payload_encoded, $signature, $key, OPENSSL_ALGO_SHA256); 
$signature_encoded = base64url_encode($signature);

echo $signature_encoded . "\n";

exec("docker run moul/drunken-bishop " . $signature_encoded,$output);

foreach($output as $line) {
    echo $line."\n";
}